# Copyright (c) 2014, 2015 Dave Love, Liverpool University
# The licence for this file is as for the package itself.

# NB  This won't work with openmpi for want of MPI_THREAD_MULTIPLE.

# Fixme: Build java_api?  (would require Bridj, at least)

%if 0%{?rhel}
%bcond_without mvapich
%endif

Name:		mdhim
Version:	1.0
Release:	2%{?dist}
Summary:	Multi-Dimensional Hashing Indexing Middleware

# See also the Mlog licence, which seems compatible
# GPL is only for the mdhimtst.c test file
License:	BSD and GPLv2+
URL:		https://github.com/mdhim/mdhim-tng
Source0:	mdhim-tng-%version.tar.gz
Source1:	mdhim-Makefile.cfg
Patch1:		mdhim-shared.patch
BuildRoot: %{_tmppath}/%{name}-%{version}-%{release}-root-%(%{__id_u} -n)

BuildRequires:	leveldb-devel mysql-devel mpich-devel mysql-connector-odbc
BuildRequires:	uthash-devel texlive-latex
%if %{with mvapich}
BuildRequires:	mvapich2-devel
%endif
Obsoletes:	libmdhim

%package mpich
Summary:	Multi-Dimensional Hashing Indexing Middleware - mpich
Requires:	mpich%{?_isa}

%global desc \
MDHIM is a parallel key/value store framework written in MPI.\
Unlike other big data solutions, MDHIM has been designed for an\
HPC environment and to take advantage of high speed networks.

%description
%desc

%description mpich
%desc

This is the mpich version.


%package mpich-devel
Summary:	Development files for %{name}-mpich
Requires:	%{name}-mpich%{?_isa} = %{version}-%{release}
# require uthash.h since uthash-devel is noarch, but appears in rpm
# requires as (x86_64), say.
Requires:	%_includedir/uthash.h mpich-devel%{?_isa}

%description mpich-devel
The %{name}-mpich-devel package contains libraries and header files for
developing applications that use %{name}-mpich.

%if %{with mvapich}
%package mvapich2
Summary:	Multi-Dimensional Hashing Indexing Middleware - mpich
Requires:	mvapich2%{?_isa}

%description mvapich2
%desc

This is the mvapich2 version.


%package mvapich2-devel
Summary:	Development files for %{name}-mvapich2
Requires:	%{name}-mvapich2%{?_isa} = %{version}-%{release}
Requires:	%_includedir/uthash.h mvapich2-devel%{?_isa}

%description mvapich2-devel
The %{name}-mvapich2-devel package contains libraries and header files for
developing applications that use %{name}-mvapich2.
%endif

%package doc
Summary:	Documentation for %{name}
BuildArch:	noarch

%description doc
Documentation for %{name}


%prep
%setup -q -n mdhim-tng-%version
%patch1 -p1 -b .shared
cp %SOURCE1 Makefile.cfg
rm -r src/uthash		# unbundled
rm java_api/*.jar
# fixme: unbundle Mlog?  Not clear if it's a coherent library,
# and can't find the original
chmod -x src/ds_mysql.[ch]


%build
%if %{with mvapich}
%_mvapich2_load
make CC=mpicc CFLAGS="-fPIC $CFLAGS -pthread -DLEVELDB_SUPPORT" \
     CINC="-IMlog" %{?_smp_mflags}
mkdir -p mvapich
mv src/libmdhim.so.* mvapich
make clean
%_mvapich2_unload
%endif
%_mpich_load
make CC=mpicc CFLAGS="-fPIC $CFLAGS -pthread -DLEVELDB_SUPPORT" \
     CINC="-IMlog" %{?_smp_mflags}
%_mpich_unload
make -C docs/latex


%install
rm -rf $RPM_BUILD_ROOT
%global doinst \
mkdir -p $RPM_BUILD_ROOT$MPI_LIB $RPM_BUILD_ROOT$MPI_INCLUDE/mdhim/Mlog\
install -m644 src/Mlog/mlog.h src/Mlog/mlogfacs.h $RPM_BUILD_ROOT$MPI_INCLUDE/mdhim/Mlog\
install -m664 src/{mdhim,data_store,range_server,messages,partitioner,mdhim_options,indexes,mdhim_private}.h $RPM_BUILD_ROOT$MPI_INCLUDE/mdhim
%_mpich_load
%doinst
install src/libmdhim.* $RPM_BUILD_ROOT$MPI_LIB
(cd  $RPM_BUILD_ROOT$MPI_LIB; ln -s libmdhim.so.* libmdhim.so)\
%_mpich_unload
%if %{with mvapich}
%_mvapich2_load
%doinst
install mvapich/libmdhim.* $RPM_BUILD_ROOT$MPI_LIB
(cd  $RPM_BUILD_ROOT$MPI_LIB; ln -s libmdhim.so.* libmdhim.so)\
%_mvapich2_unload
%endif

%clean
rm -rf $RPM_BUILD_ROOT


%if x%{?copr_username} == x
%check
%_mpich_load
make -C tests
# Set up for mysql, and can't get it working with leveldb
# ( cd tests/tester; ./runAll.sh )
# Run some random single tests instead.
cd tests/single_tests
for t in bput-bget put-get_secondary put-get puts-gets; do
  rm -rf mdhimTst*
  mpirun -np 2 ./$t
done
%_mpich_unload
%endif


%files mpich
%doc README.md LICENSE
%{_libdir}/mpich/lib/*.so.*

%files mpich-devel
%{_includedir}/mpich-%_arch/*
%{_libdir}/mpich/lib/*.so
%exclude %{_libdir}/mpich/lib/*.a

%if %{with mvapich}
%files mvapich2
%doc README.md LICENSE
%{_libdir}/mvapich2/lib/*.so.*

%files mvapich2-devel
%{_includedir}/mvapich2-%_arch/*
%{_libdir}/mvapich2/lib/*.so
%endif

%files doc
%doc README.md LICENSE docs/html docs/latex/*.pdf



%changelog
* Tue Jul  7 2015 Dave Love <d.love@liverpool.ac.uk> - 1.0-2
- Fix mvapich2-devel requires
- Make doc package

* Sat Mar  7 2015 Dave Love <d.love@liverpool.ac.uk> - 1.0-1
- New version, now shipping licence
- Install missing headers
- Modify patches
- Add tests
- Make mpich and optional mvapich packages

* Sat Jan 24 2015 Dave Love <d.love@liverpool.ac.uk> - 0.73-4
- Don't install .a

* Wed Oct 22 2014 Dave Love <d.love@liverpool.ac.uk> - 0.73-3
- Add licence and re-name package

* Tue Oct 21 2014 Dave Love <d.love@liverpool.ac.uk> - 0.73-2
- Install in the mvapich trees
- Link against leveldb
- Require uthash-devel, leveldb-devel for devel package

* Wed Jun  4 2014 Dave Love <d.love@liverpool.ac.uk> - 0.73-1
- Initial packaging
